@@ -23,10 +23,10 @@ class StateAwareMixin:
     update_log_level = logging.INFO
     
     
-    def __init__(self):
+    def __init__(self, **kw):
         'Register topics and the state callcack.'
         
-        super().__init__()
+        super().__init__(**kw)
         self.current_state = {}
 
         status_topic = self.get_config().get('status_topic')
@@ -101,9 +101,11 @@ def conditional(expression):
         
         @wraps(method)
         def wrapped(self):
+            last_state = predicate.last_state
             condition = predicate(self.current_state)
-            self.log.info('Invoking: %s(%s)', id, condition)
-            method(self, condition)
+            if condition != last_state:
+                self.log.info('Invoking: %s(%s)', id, condition)
+                method(self, condition)
         StateAwareMixin.conditions[predicate] = wrapped
         return method
     return wrapper
@@ -123,11 +125,14 @@ def when(expression):
         
         @wraps(method)
         def wrapped(self):
-            if predicate(self.current_state):
-                self.log.info('Invoking: %s', id)
-                method(self)
-            else:
-                self.log.debug('Skipping: %s', id)
+            last_state = predicate.last_state
+            condition = predicate(self.current_state)
+            if condition != last_state:
+                if condition:
+                    self.log.info('Invoking: %s', id)
+                    method(self)
+                else:
+                    self.log.debug('Skipping: %s', id)
         StateAwareMixin.conditions[predicate] = wrapped
         return method
     return wrapper
